from subprocess import call
from .arg_parser import parse_arguments
import os


def command_generator(account_feature_groups: list) -> str:
    """
    Use args, accounts, and features to construct behave command
    :param account_feature_groups:
    :return:
    """

    # get arguments
    args = parse_arguments()

    # get data for constructing behave command
    process_name = account_feature_groups[0][0]
    account = account_feature_groups[0][1].split()
    feature_set = account_feature_groups[1]

    # construct the behave command
    results_json = None
    commands = []
    for env in args.env:
        optional_retry = f'-D retry="{args.retry}" ' if args.retry > 1 else ' '
        optional_browser = f'-D browser="{args.browser}" -D headless="{args.headless}" ' if args.browser else ' '
        optional_userdata = f'-D user="{account[0]}" -D pass="{account[1]}" ' if args.account_file and args.account_section else ' '
        tags = f'--tags="{" and ".join(args.itags)} and ({" or ".join(feature_set)}){f"".join(" and not {}".format(t) for t in args.etags)}" '
        project_specific_args = (' '.join([f"-D {arg}" for arg in args.arbitrary]) if args.arbitrary else ' ') + ' '
        results_json = os.path.join(args.output_dir, f'{process_name}.json')

        cmd = (
                f'behave '
                f'-D parallel="True" ' +                # toggle parallel for any special cases in the hooks
                f'-D env="{env}" ' +                    # set the environment
                optional_retry +                        # max number of tries
                optional_browser +                      # browser (basically always chrome)
                optional_userdata +                     # optional user and password info
                tags +                                  # tags to include and exclude
                project_specific_args +                 # vary based on project (easy to break and must be of the form <something>=<something else>)
                f'-f json.pretty -o {results_json} ' +  # formatter for results
                f'-D output="{args.output_dir}" ' +     # the directory for run output
                args.feature_dir                        # feature directory
        )
        commands.append(cmd)

    # run the command(s)
    for command in commands:
        print(command, end='\n')
        r = call(command, shell=True)

    return results_json



import os.path
import sys
from . import parse_page_creator_args


def main():
    # parse arguments
    args = parse_page_creator_args.parse_arguments()

    # cancel if the page exists
    if os.path.isfile(os.path.join(args.page_object_directory, args.object_directory, f'{args.object_name}.py')):
        sys.exit('That file already exists')
    else:
        object_path = os.path.join(args.page_object_directory, args.object_directory, f'{args.object_name}.py')
        locator_path = os.path.join(args.page_object_directory, f'{args.locator_directory}', f'{args.object_name}.py')

        # create the locator file
        with open(locator_path, 'w+') as lf:
            lf.write(
                (
                    f'from {args.page_object_directory}.{args.base_locator_path.replace("/", ".")} import LocatorParent\n'
                    'from selenium.webdriver.common.by import By\n\n\n'
                    'class Locators(LocatorParent):\n'
                    '\tdef __init__(self):\n'
                    '\t\tsuper().__init__()\n'
                    '\n'
                )
            )

        # create the object file
        with open(object_path, 'w+') as of:
            of.write((
                f'from {args.page_object_directory}.{args.base_object_path.replace("/", ".")} import BasePage\n'
                f'from {args.page_object_directory}.{args.locator_directory.replace("/", ".")} import {args.object_name}\n\n\n'
                f'class Locators(BasePage):\n'
                '\tdef __init__(self, context):\n'
                '\t\tsuper().__init__(context)\n'
                f'\t\tself.locators = {args.object_name}.Locators()\n'
                '\n'
            ))


if __name__ == '__main__':
    # run everything
    main()
    sys.exit(0)



import os


def get_features(args, accounts) -> list:
    """
    Split the features as evenly as possible
    :param args:
    :param accounts:
    :return:
    """
    all_features = []

    for directory in os.walk(args.feature_dir):
        for file in directory[2]:
            if file.endswith('.feature'):
                all_features.append(f'@{args.board_name}.{file.split()[0]}')

    inc = -(-len(all_features) // args.processes)  # weird, yucky
    features = [all_features[i:i + inc] for i in range(0, len(all_features), inc)]
    return list(zip(accounts, features))



import argparse
import re

def parse_id_args():
    parser = argparse.ArgumentParser('Updating features and ids')
    parser.add_argument('--feature_path', '-fp', help='path to directory or file to add ids')
    parser.add_argument('--config_path', '-cp', help='path to directory or config')
    return parser.parse_args()


def id_generator(starting_number=0):
    while True:
        yield starting_number
        starting_number += 1


def has_id(file_name):
    pattern = re.compile(r'(?P<fid>[0-9]+).*feature')
    id_matches = re.search(pattern, file_name)
    return id_matches is not None


def is_feature(file_name):
    pattern = re.compile(r'.*(?P<extension>\.feature)')
    ext_matches = re.search(pattern, file_name)
    return ext_matches is not None



import argparse


def parse_arguments():
    parser = argparse.ArgumentParser('Running in parallel mode')

    # parallelism
    parser.add_argument('--processes',       '-p',  required=True, default=1,  type=int)
    parser.add_argument('--env',             '-e',  required=True, default=[], nargs='+')

    # dir/file paths
    parser.add_argument('--feature_dir',     '-fd', required=True, default='features')
    parser.add_argument('--output_dir',      '-od', required=True, default='output')
    parser.add_argument('--result_file',     '-rf', required=True, default='output/aggregate.json')
    parser.add_argument('--account_file',    '-af')
    parser.add_argument('--account_section', '-as')

    # direct behave arguments (every bpp QA project requires these)
    parser.add_argument('--browser',         '-br', required=False)
    parser.add_argument('--headless',        '-hd', required=False, action='store_true')
    parser.add_argument('--itags',           '-it', required=False, nargs='+')
    parser.add_argument('--etags',           '-et', required=False, default=[], nargs='+')
    parser.add_argument('--retry',           '-rt', required=False, default=1,  type=int)

    # misc and arbitrary project specific args (very breakable)
    parser.add_argument('--board_name', '-bn', required=False, default='section')
    parser.add_argument('--arbitrary',  '-ay', nargs='+')
    return parser.parse_args()



import requests
import argparse


def __parse_arguments():
    parser = argparse.ArgumentParser('Sending results to cucumber studio run')

    # params args
    parser.add_argument('--environment', '-e')
    parser.add_argument('--file_path', '-fp')

    # url args
    parser.add_argument('--cc_upload', '-cu')
    parser.add_argument('--run_id', '-ri')
    parser.add_argument('--result_format', '-rf', default='cucumber-json')

    return parser.parse_args()


def __send_results_to_test_run(environment, file_path, run_id, cc_upload_id, result_format):
    params = (
        ('execution_environment', environment),
    )
    files = {
        'file': (file_path, open(file_path, 'rb')),
    }

    response = requests.post(
        f'https://studio.cucumber.io/import_test_reports/{cc_upload_id}/{run_id}/{result_format}',
        params=params,
        files=files
    )

    print(response.json())


def main():
    args = __parse_arguments()
    __send_results_to_test_run(
        environment=args.environment,
        file_path=args.file_path,
        run_id=args.run_id,
        cc_upload_id=args.cc_upload,
        result_format=args.result_format
    )


if __name__ == '__main__':
    main()



import configparser
import os
import re
from anubis.id_generator.helpers import *


def main():
    # get args
    args = parse_id_args()

    all_features = []

    # get the last id used and increment it
    cp = configparser.ConfigParser()
    cp.read(args.config_path)
    data = cp['feature_id']
    starting_number = int(data['max'])

    id_gen = id_generator(starting_number)
    for directory in os.walk(args.feature_path):

        for file in directory[2]:
            if is_feature(file) and not has_id(file):
                file_name = os.path.join(directory[0], file)
                path = os.path.join(directory[0], file)

                with open(path) as feature_file:
                    feature_id = next(id_gen)

                    lines = feature_file.readlines()

                    lines[0] = f'@section.{feature_id} ' + int(lines[0].lstrip().startswith('Feature')) * '\n' + lines[0]

                    # feature_file.writelines(lines)
                with open(path, 'w+') as feature_file:
                    feature_file.writelines(lines)

                cp.set('feature_id', 'max', str(feature_id+1))
                cp.write(open(args.config_path, 'w'))

                new_file_name = os.path.join(directory[0], f'{feature_id} {file}')
                # print(f'os.rename({file_name}, {new_file_name})')
                os.rename(file_name, new_file_name)
            else:
                print(f'Skipping: <{file}>')

if __name__ == '__main__':
    main()



import configparser
import sys
import os


def get_accounts(num_processes: int, accounts_file_path: str, accounts_section: str) -> list:
    """
    Reads the `accounts_sections` in the `accounts_file_path` file and return list of accounts
    :param num_processes:
    :param accounts_file_path:
    :param accounts_section:
    :return:
    """

    # try to read the accounts file and create parser
    data = configparser.ConfigParser()
    accounts_data = None
    if os.path.isfile(accounts_file_path):
        data.read(accounts_file_path)
    else:
        print(f'Cannot find this file: <{accounts_file_path}>\nExiting without running tests!')
        sys.exit(1)

    # get the accounts from the parser
    try:
        accounts_data = data[accounts_section]
    except KeyError:
        print(f'Could not find this section: <{accounts_section}>')
        sys.exit(1)

    # return a list of the accounts, if possible
    if num_processes > len(list(accounts_data.values())):
        print("Not enough accounts; Cannot run tests")
        sys.exit(1)
    else:
        return [acc for acc in list(accounts_data.items())]



"""___anubis - Framework for running bpp qa"""
from . import account_splitter, arg_parser, feature_splitter, parallelizer, results
__version__ = '0.1.0'
__author__ = 'matthew bahloul <matthew.bahloul@blueprintprep.com>'
__all__ = ['account_splitter', 'arg_parser', 'feature_splitter', 'parallelizer', 'results']



import json
import zipfile
import os


def create_aggregate(files: list, aggregate_out_file):
    aggregate = []
    for fp in files:
        with open(fp, 'r') as f:
            aggregate += json.load(f)
    with open(aggregate_out_file, 'w+') as f:
        f.write(json.dumps(aggregate))


def zipper(dir_to_zip):
    best_zipper_ever = zipfile.ZipFile(f'{dir_to_zip}.zip', 'w', zipfile.ZIP_DEFLATED)
    for root, dirs, files in os.walk(dir_to_zip):
        for file in files:
            best_zipper_ever.write(
                os.path.join(root, file),
                os.path.relpath(
                    os.path.join(root, file),
                    os.path.join(dir_to_zip, '../..')
                )
            )

    best_zipper_ever.close()


def reformat(form: str):
    if form.lower().replace(' ', '').replace('-', '') == 'testrail':
        print('reformat for testrail')
    raise NotImplementedError



import requests
import json
import argparse
import os
from datetime import datetime


def __parse_arguments():
    current_date_time = datetime.now().strftime('[%y/%m/%d][%H:%M %p]')
    parser = argparse.ArgumentParser('Creating run')

    # header args
    parser.add_argument('--content-type',    '-ct', default="application/json; charset=utf-8")
    parser.add_argument('--accept',          '-a', default="application/vnd.api+json; version=1")
    parser.add_argument('--access_token',    '-at', required=True)
    parser.add_argument('--client_id',       '-ci', required=True)
    parser.add_argument('--unique_id',       '-ui', required=True)

    # data args
    parser.add_argument('--run_name',        '-rn', default=current_date_time)
    parser.add_argument('--run_description', '-rd', default=f'Run created on {current_date_time}')
    parser.add_argument('--project_id',      '-pi', required=True)

    return parser.parse_args()


def create_test_run(project_id, access_token, client_id, uid, name, description):

    headers = {
        "Content-Type": "application/json; charset=utf-8",
        "Accept": "application/vnd.api+json; version=1",
        "access-token": access_token,
        "client": client_id,
        "uid": uid,
    }

    data = json.dumps({
        "data": {
            "attributes": {
                "name": name,
                "description": description,
            }
        }
    })

    return requests.post(
        f"https://studio.cucumber.io/api/projects/{project_id}/test_runs",
        headers=headers,
        data=data
    )


def main():
    args = __parse_arguments()

    res = create_test_run(
        project_id=args.project_id,
        access_token=args.access_token,
        client_id=args.client_id,
        uid=args.unique_id,
        name=args.run_name,
        description=args.run_description
    )

    print(res.json()["data"]["id"])


if __name__ == '__main__':
    main()



import argparse


def parse_arguments():
    parser = argparse.ArgumentParser('Creating Object')

    parser.add_argument('--page_object_directory', '-pd', default='page_objects',  help='The directory with all the page/component objects')
    parser.add_argument('--base_object_path',      '-bo', default='_base_page',    help='The file name of the page parent')
    parser.add_argument('--base_locator_path',     '-bl', default='_base_locator', help='The file name of the locator parent')
    parser.add_argument('--object_name',           '-n',  required=True,           help='Name of the new object file (no extension)')
    parser.add_argument('--object_directory',      '-o',  required=True,           help='Directory to save new object file')
    parser.add_argument('--locator_directory',     '-l',  required=True,           help='Directory to save locator for new object')

    return parser.parse_args()



import io
import os
import re

from setuptools import find_packages
from setuptools import setup


def read(filename):
    filename = os.path.join(os.path.dirname(__file__), filename)
    text_type = type(u"")
    with io.open(filename, mode="r", encoding='utf-8') as fd:
        return re.sub(text_type(r':[a-z]+:`~?(.*?)`'), text_type(r'``\1``'), fd.read())


setup(
    name="bpp-anubis",
    version="0.2.45",
    url="https://tools.blueprintlsat.com/qa/anubis",
    license='MIT',

    author="matthew bahloul",
    author_email="matthew.bahloul@blueprintprep.com",

    description="Framework for running bpp qa",
    long_description=read("README.rst"),

    packages=find_packages(exclude=('tests',)),

    install_requires=[],

    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    entry_points={
            'console_scripts': [
                'anubis = anubis.__main__:main',
                'anubis-create-cc-run = anubis.sync.cucumber_studio.create_run:main',
                'anubis-send-cc-results = anubis.sync.cucumber_studio.send_results:main',
                'anubis-generate-ids = anubis.id_generator.__main__:main',
                'anubis-mkobj = anubis.page_creator.__main__:main'
            ],
        }
)



# __main__.py
import os
import json
import multiprocessing
from multiprocessing import Pool
import sys
from datetime import datetime

# custom
from . import account_splitter, feature_splitter, arg_parser, results
from .parallelizer import command_generator

ANUBIS_ASCII = (
    """
                         ♡♡                                               
                         ♡♡♡                                                 
                       ♡♡ ♡♡♡♡                                               
                  ♡♡♡♡♡♡♡♡♡♡♡♡                                               
                          ♡♡♡♡♡                                              
                           ♡♡♡♡♡                                             
                        ♡♡♡♡♡♡♡♡♡                                            
                       ♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡        ♡♡♡♡♡♡                         
                       ♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡ ♡♡♡♡                     
                       ♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡                   
                   ♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡     ♡♡♡♡♡♡♡♡♡♡                  
         ♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡       ♡♡♡♡♡     ♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡                 
                                                             ♡♡♡              
                   POWERED BY ANUBIS                          ♡♡♡♡            
         (and the power of love  Σ>―(〃°ω°〃)♡→)               ♡♡♡♡♡          
                                                               ♡♡♡♡♡         
                                                                ♡♡♡♡         
                                                                 ♡♡     
    """
)


def main():
    print(ANUBIS_ASCII)
    start = datetime.now()

    # parse arguments
    args = arg_parser.parse_arguments()

    # create a temp dir that will contain results and be exported
    if not os.path.isdir(args.output_dir):
        # todo - think about whether this is the responsibility of this tool
        print(f'Could not find directory for output: <{args.output_dir}>\nCreating directory <{args.output_dir}> now')
        os.mkdir(args.output_dir)
        # sys.exit(1)

    # set up the multiple processes
    multiprocessing.set_start_method('fork')
    pool = Pool(args.processes)

    # get account data available for parallel runs
    # the `accounts_data` list looks like this --> [(<run_name>, "user pass"), [<list of feature files>]]
    if args.account_file and args.account_section:
        print('\n--- PARSING ACCOUNTS')
        print(f'\tfile:    <{args.account_file}>')
        print(f'\tsection: <{args.account_section}>')
        accounts_data = account_splitter.get_accounts(args.processes, args.account_file, args.account_section)
    else:
        print('\n--- ACCOUNTS NOT SPECIFIED')
        # create dummy account data just for the purpose of naming the runs
        accounts_data = [(datetime.now().isoformat(), 'NONE NONE') for i in range(args.processes)]

    # split the features and store as list
    print('\n--- GROUPING FEATURES & ACCOUNTS')
    print(f'\tfeature dir:   <{args.feature_dir}>')
    print(f'\tincluded tags: <{",".join([t for t in args.itags]) if args.itags else "(none)"}>')
    print(f'\texcluded tags: <{",".join([t for t in args.etags]) if args.etags else "(none)"}>')
    feature_groups = feature_splitter.get_features(args, accounts_data)

    # run all the processes and save the locations of the result files
    num_groups = len(feature_groups)
    print(f'\n--- RUNNING <{num_groups} PROCESS{"ES" * int(num_groups > 1)}>')
    result_files = pool.map(command_generator, feature_groups)

    # recombine everything
    print('\n--- COMBINING RESULTS')
    try:
        results.create_aggregate(files=result_files, aggregate_out_file=args.result_file)
    except Exception as e:
        print(f'There was an error combining results\n{e}')

    end = datetime.now()

    # calculate passes/fails for quick summary
    res_string = None
    passes, failures, other = 0, 0, 0
    try:
        with open(args.result_file) as f:
            res = json.load(f)
        for feature in res:
            for scenario in feature['elements']:
                if scenario['keyword'].lower() != 'background':
                    status = scenario['status']
                    passes += 1 * int(status == 'passed')
                    failures += 1 * int(status == 'failed')
                    other += 1 * int(status != 'passed' and status != 'failed')
        res_string = f'{passes / (passes + failures) * 100:.2f}%'
    except Exception as e:
        # I know that this is a terrible anti-pattern, but there's really nothing to do for an exception
        print(f"\tThere was an error calculating overall results, but it's likely not fatal.\n\t{e}")

    # extremely basic summary
    print('\n=======================================\n\t\tSUMMARY\n=======================================')
    print(f'Env(s):    <{",".join(args.env)}>')
    print(f'Browser:   <{args.browser}>')
    print(f'Results:   <{args.result_file}>')
    print(f'Pass Rate: <{res_string if res_string else "could not calculate"}>')
    print(f'Run Time:  <{(end - start)}>')
    print('=======================================')


if __name__ == '__main__':
    # run everything
    main()
    sys.exit(0)
